@@ -198,6 +198,7 @@ main( int argc, char **argv )
 	int width = -1;
 	int height = -1;
 	int dpi = -1;
+	int max = 0;
 	int one2one = 0;
 	PrinterGeometry *geo = find_printer( "2500cp" );
 	char *mode;
@@ -287,7 +288,7 @@ main( int argc, char **argv )
 			else if( strcmp( argv[i]+1, "a0" ) == 0 ) 
 				height = 3356;
 			else if( strcmp( argv[i]+1, "max" ) == 0 ) 
-				width = geo->width;
+				max = 1;
 			else
 				error_exit( "bad flag" );
 		}
@@ -332,13 +333,25 @@ main( int argc, char **argv )
 		height = (((im->Ysize / im->Yres) / 10.0) / 2.54) * 72.0;
 		width = (((im->Xsize / im->Xres) / 10.0) / 2.54) * 72.0;
 	}
+	else if( max ) {
+		float iaspect = (float) im->Xsize / im->Ysize;
+		float paspect = (float) geo->width / geo->length;
+
+		if( iaspect > paspect ) 
+			/* Image aspect ratio > paper ... fit width.
+			 */
+			width = geo->width;
+		else
+			height = geo->length;
+	}
 	else if( dpi > 0 ) {
 		/* Given res ... set width/height.
 		 */
 		height = (im->Ysize / (float) dpi) * 72.0;
 		width = (im->Xsize / (float) dpi) * 72.0;
 	}
-	else if( width >= 0 || height >= 0 ) {
+
+	if( width >= 0 || height >= 0 ) {
 		/* Given width or height or both --- set other one.
 		 */
 		if( height < 0 ) {
