@@ -13,12 +13,14 @@
  * 8/4/99 JC
  *	- lots of consts added to please C++
  *	- and more protos added
+ * 11/9/06
+ * 	- internal protos cut out to help SWIG
  */
 
 /*
 
     This file is part of VIPS.
-    
+
     VIPS is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
@@ -53,14 +55,24 @@ extern "C" {
 #include <stdio.h>
 #include <stdarg.h>
 #include <sys/types.h>
-#include "rect.h"
+#include <glib-object.h>
+
+/* If we're being parsed by SWIG, remove gcc attributes.
+ */
+#ifdef SWIG
+#  ifndef __attribute__
+#    define __attribute__(x)  /*NOTHING*/
+#  endif
+#endif /*SWIG*/
 
 typedef int (*im_callback_fn)( void *, void * );
 typedef void *(*im_construct_fn)( void *, void *, void * );
+typedef void *(*im_header_map_fn)( IMAGE *, const char *, GValue *, void * );
 
 /* iofuncs
  */
 int im_init_world( const char *argv0 );
+GOptionGroup *im_get_option_group( void );
 
 const char *im_error_buffer( void );
 int im_debugim( IMAGE * );
@@ -69,6 +81,10 @@ int im_printlines( IMAGE * );
 int im_header_int( IMAGE *im, const char *field, int *out );
 int im_header_double( IMAGE *im, const char *field, double *out );
 int im_header_string( IMAGE *im, const char *field, char **out );
+GType im_header_get_type( IMAGE *im, const char *field );
+int im_header_get( IMAGE *im, const char *field, GValue *value_copy );
+void *im_header_map( IMAGE *im, im_header_map_fn fn, void *a );
+
 const char *im_version_string( void );
 int im_version( int flag );
 const char *im_guess_prefix( const char *, const char * );
@@ -89,7 +105,7 @@ IMAGE *im_open( const char *, const char * );
 IMAGE *im_open_header( const char * );
 int im_image_sanity( IMAGE * );
 
-void *im_malloc( IMAGE *im, int sz );
+void *im_malloc( IMAGE *im, size_t sz );
 int im_free( void * );
 
 int im_close( IMAGE * );
@@ -101,14 +117,16 @@ int im_piocheck( IMAGE *, IMAGE * );
 int im_pincheck( IMAGE * );
 int im_poutcheck( IMAGE * );
 int im_cp_desc( IMAGE *, IMAGE * );
-int im_cp_Hist( IMAGE *, IMAGE * );
-int im_append_Hist( IMAGE *, IMAGE * );
+int im_cp_descv( IMAGE *out, IMAGE *in1, ... )
+	__attribute__((sentinel));
+int im_cp_desc_array( IMAGE *out, IMAGE *in[] );
 int im_setupout( IMAGE * );
 int im_writeline( int, IMAGE *, PEL * );
 
 int im_isuint( IMAGE * );
 int im_isint( IMAGE * );
 int im_isfloat( IMAGE * );
+int im_isscalar( IMAGE * );
 int im_iscomplex( IMAGE * );
 int im_isfile( IMAGE * );
 int im_ispartial( IMAGE * );
@@ -117,29 +135,37 @@ int im_amiMSBfirst( void );
 
 int im_ispoweroftwo( int );
 
-int im_existsf( const char *name, ... );
+int im_existsf( const char *name, ... )
+	__attribute__((format(printf, 1, 2)));
 int im_istiff( const char * );
 int im_istifftiled( const char * );
 int im_istiffpyramid( const char * );
 int im_isjpeg( const char * );
 int im_isvips( const char * );
+int im_isexr( const char * );
 int im_isppm( const char * );
 int im_ispng( const char * );
 int im_ismagick( const char * );
+int im_isanalyze( const char *filename );
 
 int im_add_close_callback( IMAGE *, im_callback_fn, void *, void * );
 int im_add_eval_callback( IMAGE *, im_callback_fn, void *, void * );
 int im_add_evalend_callback( IMAGE *, im_callback_fn, void *, void * );
 
-void error_exit( const char *, ... );
+void error_exit( const char *, ... )
+	__attribute__((noreturn, format(printf, 1, 2)));
 void im_error_clear( void );
 void im_verror( const char *domain, const char *fmt, va_list ap );
-void im_error( const char *domain, const char *fmt, ... );
-void im_error_system( int err, const char *domain, const char *fmt, ... );
-void im_warn( const char *domain, const char *fmt, ... );
-void im_diag( const char *domain, const char *fmt, ... );
+void im_error( const char *domain, const char *fmt, ... )
+	__attribute__((format(printf, 2, 3)));
+void im_error_system( int err, const char *domain, const char *fmt, ... )
+	__attribute__((format(printf, 3, 4)));
+void im_warn( const char *domain, const char *fmt, ... )
+	__attribute__((format(printf, 2, 3)));
+void im_diag( const char *domain, const char *fmt, ... )
+	__attribute__((format(printf, 2, 3)));
 
-void im_printhd( VAS_HD * );
+int im_bits_of_fmt( int );
 const char *im_Type2char( int );
 const char *im_BandFmt2char( int );
 const char *im_Coding2char( int );
@@ -153,26 +179,19 @@ int im_char2Compression( const char * );
 
 int im_unmapfile( IMAGE * );
 void im_printdesc( IMAGE * );
-void im_initdesc( IMAGE *, 
+void im_initdesc( IMAGE *,
 	int, int, int, int, int, int, int, float, float,
 	int, int );
-void im_inithd( VAS_HD *hd, int bnd, int bndbits, int bndfmt,
-	int xsize, int ysize,
-	int imtype, int codfmt, double xres, double yres,
-	int Length, int Compression, int Level,
-	int Xoffset, int Yoffset );
-int im_desc_hd( IMAGE *image, VAS_HD *header );
-int im_readhist( IMAGE *image );
-int im_crwrhd( IMAGE *image );
-int im_histlin( IMAGE *image, const char *fmt, ... );
-int im_updatehist( IMAGE *out, int argc, char *argv[] );
+int im_histlin( IMAGE *image, const char *fmt, ... )
+	__attribute__((format(printf, 2, 3)));
+int im_updatehist( IMAGE *out, const char *name, int argc, char *argv[] );
+const char *im_history_get( IMAGE *im );
 
-extern int im__read_test;
-int im_render( IMAGE *in, IMAGE *out, IMAGE *mask, 
-	int width, int height, int max, 
+int im_render( IMAGE *in, IMAGE *out, IMAGE *mask,
+	int width, int height, int max,
 	void (*notify)( IMAGE *, Rect *, void * ), void *client );
 int im_render_fade( IMAGE *in, IMAGE *out, IMAGE *mask,
-	int width, int height, int max, 
+	int width, int height, int max,
 	int fps, int steps,
 	int priority,
 	void (*notify)( IMAGE *, Rect *, void * ), void *client );
@@ -208,6 +227,7 @@ int im_write_dmask( DOUBLEMASK * );
 int im_write_imask_name( INTMASK *, const char * );
 int im_write_dmask_name( DOUBLEMASK *, const char * );
 INTMASK *im_scale_dmask( DOUBLEMASK *, const char * );
+void im_norm_dmask( DOUBLEMASK *mask );
 int *im_offsets45( int );
 int *im_offsets90( int );
 INTMASK *im_rotate_imask90( INTMASK *, const char * );
@@ -229,11 +249,8 @@ int im_zerox( IMAGE *, IMAGE *, int );
 int im_maxvalue( IMAGE **in, IMAGE *out, int n );
 int im_rank_image( IMAGE **in, IMAGE *out, int n, int index );
 int im_compass( IMAGE *, IMAGE *, INTMASK * );
-int im_compass_old( IMAGE *, IMAGE *, INTMASK * );
 int im_gradient( IMAGE *, IMAGE *, INTMASK * );
-int im_gradient_old( IMAGE *, IMAGE *, INTMASK * );
 int im_lindetect( IMAGE *, IMAGE *, INTMASK * );
-int im_lindetect_old( IMAGE *, IMAGE *, INTMASK * );
 int im_conv( IMAGE *, IMAGE *, INTMASK * );
 int im_conv_raw( IMAGE *, IMAGE *, INTMASK * );
 int im_convf( IMAGE *, IMAGE *, DOUBLEMASK * );
@@ -244,10 +261,19 @@ int im_convsepf( IMAGE *, IMAGE *, DOUBLEMASK * );
 int im_convsepf_raw( IMAGE *, IMAGE *, DOUBLEMASK * );
 int im_convsub( IMAGE *, IMAGE *, INTMASK *, int, int );
 
+int im_grad_x( IMAGE *in, IMAGE *out );
+int im_grad_y( IMAGE *in, IMAGE *out );
+
 int im_fastcor( IMAGE *, IMAGE *, IMAGE * );
 int im_fastcor_raw( IMAGE *, IMAGE *, IMAGE * );
 int im_spcor( IMAGE *, IMAGE *, IMAGE * );
 int im_spcor_raw( IMAGE *, IMAGE *, IMAGE * );
+int im_spcor2( IMAGE *, IMAGE *, IMAGE * );
+int im_spcor2_raw( IMAGE *, IMAGE *, IMAGE * );
+int im_gradcor( IMAGE *, IMAGE *, IMAGE * );
+int im_gradcor_raw( IMAGE *, IMAGE *, IMAGE * );
+int im_contrast_surface( IMAGE *, IMAGE *, int, int );
+int im_contrast_surface_raw( IMAGE *, IMAGE *, int, int );
 
 int im_resize_linear( IMAGE *, IMAGE *, int, int );
 int im_mpercent( IMAGE *, double, int * );
@@ -269,7 +295,7 @@ int im_invfftr( IMAGE *, IMAGE * );
 
 /* arithmetic
  */
-DOUBLEMASK *im_measure( IMAGE *, IMAGE_BOX *, 
+DOUBLEMASK *im_measure( IMAGE *, IMAGE_BOX *,
 	int, int, int *, int, const char * );
 DOUBLEMASK *im_stats( IMAGE * );
 int im_abs( IMAGE *in, IMAGE *out );
@@ -279,13 +305,18 @@ int im_avg( IMAGE *in, double *out );
 int im_deviate( IMAGE *in, double *out );
 int im_maxpos( IMAGE *in, int *xpos, int *ypos, double *out );
 int im_minpos( IMAGE *in, int *xpos, int *ypos, double *out );
+int im_maxpos_avg( IMAGE *im, double *xpos, double *ypos, double *out );
+int im_maxpos_vec( IMAGE *im, int *xpos, int *ypos, double *maxima, int n );
+int im_minpos_vec( IMAGE *im, int *xpos, int *ypos, double *minima, int n );
 int im_add( IMAGE *, IMAGE *, IMAGE * );
 int im_subtract( IMAGE *, IMAGE *, IMAGE * );
 int im_invert( IMAGE *, IMAGE * );
+int im_linreg( IMAGE **ins, IMAGE *out, double *xs );
 int im_lintra( double, IMAGE *, double, IMAGE * );
 int im_lintra_vec( int n, double *a, IMAGE *in, double *b, IMAGE *out );
 int im_multiply( IMAGE *, IMAGE *, IMAGE * );
 int im_divide( IMAGE *, IMAGE *, IMAGE * );
+int im_point_bilinear( IMAGE *im, double x, double y, int band, double *val );
 int im_powtra( IMAGE *, IMAGE *, double );
 int im_powtra_vec( IMAGE *in, IMAGE *out, int n, double *e );
 int im_exptra( IMAGE *, IMAGE * );
@@ -311,6 +342,7 @@ int im_cmulnorm( IMAGE *, IMAGE *, IMAGE * );
 int im_fav4( IMAGE **, IMAGE * );
 int im_gadd( double, IMAGE *, double, IMAGE *, double, IMAGE *);
 int im_litecor( IMAGE *, IMAGE *, IMAGE *, int, double );
+int im_bandmean( IMAGE *in, IMAGE *out );
 
 /* boolean
  */
@@ -345,6 +377,7 @@ int im_identity_ushort( IMAGE *lut, int bands, int sz );
 int im_lhisteq( IMAGE *in, IMAGE *out, int xwin, int ywin );
 int im_lhisteq_raw( IMAGE *in, IMAGE *out, int xwin, int ywin );
 int im_invertlut( DOUBLEMASK *input, IMAGE *output, int lut_size );
+int im_buildlut( DOUBLEMASK *input, IMAGE *output );
 int im_stdif( IMAGE *in, IMAGE *out,
 	double a, double m0, double b, double s0, int xwin, int ywin );
 int im_stdif_raw( IMAGE *in, IMAGE *out,
@@ -360,24 +393,40 @@ int im_tone_analyse( IMAGE *in, IMAGE *lut,
 	double Ps, double Pm, double Ph, double S, double M, double H );
 int im_ismonotonic( IMAGE *lut, int *out );
 int im_tone_map( IMAGE *in, IMAGE *out, IMAGE *lut );
+int im_project( IMAGE *in, IMAGE *hout, IMAGE *vout );
 
 /* conversion
  */
+
+/* Copy and swap types.
+ */
+typedef enum {
+	IM_ARCH_NATIVE,
+	IM_ARCH_BYTE_SWAPPED,
+	IM_ARCH_LSB_FIRST,
+	IM_ARCH_MSB_FIRST
+} im_arch_type;
+
 DOUBLEMASK *im_vips2mask( IMAGE *, const char * );
 int im_mask2vips( DOUBLEMASK *, IMAGE * );
 int im_copy_set( IMAGE *, IMAGE *, int, float, float, int, int );
+int im_copy_set_meta( IMAGE *in, IMAGE *out, const char *field, GValue *meta );
+int im_copy_morph( IMAGE *, IMAGE *, int, int, int );
 int im_copy( IMAGE *, IMAGE * );
 int im_copy_swap( IMAGE *in, IMAGE *out );
+int im_copy_from( IMAGE *in, IMAGE *out, im_arch_type architecture );
 int im_extract( IMAGE *, IMAGE *, IMAGE_BOX * );
 int im_extract_band( IMAGE *in, IMAGE *out, int band );
 int im_extract_bands( IMAGE *in, IMAGE *out, int band, int nbands );
 int im_extract_area( IMAGE *in, IMAGE *out, int x, int y, int w, int h );
+int im_extract_areabands( IMAGE *in, IMAGE *out,
+	int left, int top, int width, int height, int band, int nbands );
 int im_subsample( IMAGE *, IMAGE *, int, int );
 int im_zoom( IMAGE *, IMAGE *, int, int );
 int im_bandjoin( IMAGE *, IMAGE *, IMAGE * );
 int im_gbandjoin( IMAGE **, IMAGE *, int );
 int im_black( IMAGE *, int, int, int );
-int im_text( IMAGE *out, const char *text, const char *font, 
+int im_text( IMAGE *out, const char *text, const char *font,
 	int width, int alignment, int dpi );
 int im_c2amph( IMAGE *, IMAGE * );
 int im_c2rect( IMAGE *, IMAGE * );
@@ -422,17 +471,28 @@ int im_vips2tiff( IMAGE *, const char * );
 int im_bernd( const char *, int, int, int, int );
 int im_tiff2vips( const char *, IMAGE * );
 int im_tiff2vips_header( const char *, IMAGE * );
+int im_tile_cache( IMAGE *, IMAGE *, int, int, int );
 int im_magick2vips( const char *, IMAGE * );
 int im_magick2vips_header( const char *, IMAGE * );
 int im_png2vips( const char *, IMAGE * );
 int im_png2vips_header( const char *, IMAGE * );
+int im_exr2vips( const char *, IMAGE * );
+int im_exr2vips_header( const char *, IMAGE * );
 int im_ppm2vips( const char *, IMAGE * );
 int im_ppm2vips_header( const char *, IMAGE * );
 int im_vips2ppm( IMAGE *, const char * );
-int im_dicom2vips( const char *, IMAGE * );
-int im_dicom2vips_header( const char *, IMAGE * );
+int im_analyze2vips( const char *filename, IMAGE *out );
+int im_analyze2vips_header( const char *filename, IMAGE *out );
+int im_vips2csv( IMAGE *in, const char *filename );
+int im_csv2vips( const char *filename, IMAGE *out );
+int im_csv2vips_header( const char *filename, IMAGE *out );
 int im_vips2png( IMAGE *, const char * );
+int im_raw2vips( const char *filename, IMAGE *out,
+	int width, int height, int bpp, int offset );
 int im_replicate( IMAGE *in, IMAGE *out, int across, int down );
+int im_grid( IMAGE *in, IMAGE *out, int tile_height, int across, int down );
+int im_msb ( IMAGE * in, IMAGE * out );
+int im_msb_band ( IMAGE * in, IMAGE * out, int band );
 
 /* colour
  */
@@ -467,6 +527,7 @@ int im_dE_fromLab( IMAGE *, IMAGE *, IMAGE * );
 
 void imb_Lab2LCh( float *, float *, int );
 void imb_LCh2Lab( float *, float *, int );
+void imb_XYZ2Lab_tables( void );
 void imb_XYZ2Lab( float *, float *, int, im_colour_temperature * );
 void imb_Lab2XYZ( float *, float *, int, im_colour_temperature * );
 void imb_LabQ2Lab( PEL *, float *, int );
@@ -480,30 +541,32 @@ float im_col_dECMC( float, float, float, float, float, float );
 float im_col_dE00( float, float, float, float, float, float );
 
 int im_lab_morph( IMAGE *in, IMAGE *out,
-	DOUBLEMASK *mask, 
-	double L_offset, double L_scale, 
+	DOUBLEMASK *mask,
+	double L_offset, double L_scale,
 	double a_scale, double b_scale );
 
 /* other
  */
-int im_feye( IMAGE *image, 
+int im_feye( IMAGE *image,
 	const int xsize, const int ysize, const double factor );
-int im_eye( IMAGE *image, 
+int im_eye( IMAGE *image,
 	const int xsize, const int ysize, const double factor );
 int im_zone( IMAGE *im, int size );
 int im_fzone( IMAGE *im, int size );
 int im_grey( IMAGE *im, const int xsize, const int ysize );
 int im_fgrey( IMAGE *im, const int xsize, const int ysize );
 int im_make_xy( IMAGE *out, const int xsize, const int ysize );
+int im_benchmarkn( IMAGE *in, IMAGE *out, int n );
+int im_benchmark2( IMAGE *in, double *out );
 
-int im_cooc_matrix( IMAGE *im, IMAGE *m, 
+int im_cooc_matrix( IMAGE *im, IMAGE *m,
 	int xp, int yp, int xs, int ys, int dx, int dy, int flag );
 int im_cooc_asm( IMAGE *m, double *asmoment );
 int im_cooc_contrast( IMAGE *m, double *contrast );
 int im_cooc_correlation( IMAGE *m, double *correlation );
 int im_cooc_entropy( IMAGE *m, double *entropy );
 
-int im_glds_matrix( IMAGE *im, IMAGE *m, 
+int im_glds_matrix( IMAGE *im, IMAGE *m,
 	int xpos, int ypos, int xsize, int ysize, int dx, int dy );
 int im_glds_asm( IMAGE *m, double *asmoment );
 int im_glds_contrast( IMAGE *m, double *contrast );
@@ -511,23 +574,25 @@ int im_glds_entropy( IMAGE *m, double *entropy );
 int im_glds_mean( IMAGE *m, double *mean );
 
 int im_simcontr( IMAGE *image, int xs, int ys );
-int im_sines( IMAGE *image, 
+int im_sines( IMAGE *image,
 	int xsize, int ysize, double horfreq, double verfreq );
 int im_spatres( IMAGE *in,  IMAGE *out, int step );
 
+int im_rightshift_size( IMAGE *in, IMAGE *out, int xshift, int yshift, int band_fmt );
+
 /* mosaicing
  */
-int im_lrmerge( IMAGE *ref, IMAGE *sec, IMAGE *out, 
+int im_lrmerge( IMAGE *ref, IMAGE *sec, IMAGE *out,
 	int dx, int dy, int mwidth );
-int im_tbmerge( IMAGE *ref, IMAGE *sec, IMAGE *out, 
+int im_tbmerge( IMAGE *ref, IMAGE *sec, IMAGE *out,
 	int dx, int dy, int mwidth );
 
 int im_lrmerge1( IMAGE *ref, IMAGE *sec, IMAGE *out,
-	int xr1, int yr1, int xs1, int ys1, 
+	int xr1, int yr1, int xs1, int ys1,
 	int xr2, int yr2, int xs2, int ys2,
 	int mwidth );
 int im_tbmerge1( IMAGE *ref, IMAGE *sec, IMAGE *out,
-	int xr1, int yr1, int xs1, int ys1, 
+	int xr1, int yr1, int xs1, int ys1,
 	int xr2, int yr2, int xs2, int ys2,
 	int mwidth );
 
@@ -546,14 +611,14 @@ int im_tbmosaic( IMAGE *ref, IMAGE *sec, IMAGE *out,
 
 int im_lrmosaic1( IMAGE *ref, IMAGE *sec, IMAGE *out,
 	int bandno,
-	int xr1, int yr1, int xs1, int ys1, 
+	int xr1, int yr1, int xs1, int ys1,
 	int xr2, int yr2, int xs2, int ys2,
 	int halfcorrelation, int halfarea,
 	int balancetype,
 	int mwidth );
 int im_tbmosaic1( IMAGE *ref, IMAGE *sec, IMAGE *out,
 	int bandno,
-	int xr1, int yr1, int xs1, int ys1, 
+	int xr1, int yr1, int xs1, int ys1,
 	int xr2, int yr2, int xs2, int ys2,
 	int halfcorrelation, int halfarea,
 	int balancetype,
@@ -578,11 +643,11 @@ int im_similarity( IMAGE *in, IMAGE *out,
 int im_similarity_area( IMAGE *in, IMAGE *out,
 	double a, double b, double dx, double dy,
 	int ox, int oy, int ow, int oh );
-int im_correl( IMAGE *ref, IMAGE *sec, 
+int im_correl( IMAGE *ref, IMAGE *sec,
 	int xref, int yref, int xsec, int ysec,
 	int hwindowsize, int hsearchsize,
 	double *correlation, int *x, int *y );
-int im_remosaic( IMAGE *in, IMAGE *out, 
+int im_remosaic( IMAGE *in, IMAGE *out,
 	const char *old_str, const char *new_str );
 
 /* inplace
@@ -598,6 +663,9 @@ int im_fastlineuser();
 int im_readpoint( IMAGE *, int, int, PEL * );
 int im_flood( IMAGE *, int, int, PEL *, Rect * );
 int im_flood_blob( IMAGE *, int, int, PEL *, Rect * );
+int im_flood_blob_copy( IMAGE *in, IMAGE *out, int x, int y, PEL *ink );
+int im_lineset( IMAGE *in, IMAGE *out, IMAGE *mask, IMAGE *ink,
+	int n, int *x1v, int *y1v, int *x2v, int *y2v );
 
 /* relational
  */
@@ -624,11 +692,16 @@ int im_blend( IMAGE *, IMAGE *, IMAGE *, IMAGE * );
 
 /* matrix
  */
-DOUBLEMASK *im_matinv( DOUBLEMASK *, const char * );
 DOUBLEMASK *im_mattrn( DOUBLEMASK *, const char * );
 DOUBLEMASK *im_matcat( DOUBLEMASK *, DOUBLEMASK *, const char * );
 DOUBLEMASK *im_matmul( DOUBLEMASK *, DOUBLEMASK *, const char * );
 
+DOUBLEMASK *im_lu_decomp( const DOUBLEMASK *mat, const char *name );
+int im_lu_solve( const DOUBLEMASK *lu, double *vec );
+DOUBLEMASK *im_matinv( const DOUBLEMASK *mat, const char *name );
+int im_matinv_inplace( DOUBLEMASK *mat );
+
+
 int *im_ivector();
 float *im_fvector();
 double *im_dvector();
@@ -648,55 +721,9 @@ int im_invmat( double **, int );
 /* video
  */
 int im_video_v4l1( IMAGE *im, const char *device,
-	int channel, int brightness, int colour, int contrast, int hue, 
+	int channel, int brightness, int colour, int contrast, int hue,
 	int ngrabs );
-
-/* Internal.
- */
-int im__cast_and_call();
-const char *im__gettext( const char *msgid );
-const char *im__ngettext( const char *msgid, 
-	const char *plural, unsigned long int n );
-int im__read_header( IMAGE *image );
-int im__test_kill( IMAGE *im );
-void *im__mmap( int fd, int writeable, size_t length, gint64 offset );
-int im__munmap( void *start, size_t length );
-int im__write( int, const void *, size_t );
-int im__open_image_file( const char *filename );
-void im__change_suffix( const char *name, char *out, int mx,
-        const char *new_suff, const char **olds, int nolds );
-void im__print_all( void );
-void im__print_one( int );
-int im__trigger_callbacks( GSList *cblist );
-int im__close( IMAGE * );
-int im__handle_eval( IMAGE *im, int w, int h );
-int im__create_int_luts( int *, int, int **, int **, int * );
-int im__create_double_luts( double *, int, double **, double **, int * );
-int im__fft_sp( float *rvec, float *ivec, int logrows, int logcols );
-typedef int (*im__fftproc_fn)( IMAGE *, IMAGE *, IMAGE * );
-int im__fftproc( IMAGE *dummy, IMAGE *in, IMAGE *out, im__fftproc_fn fn );
-int im__mean_std_double_buffer( double *buffer, int size,
-	double *pmean, double *pstd );
-int im__mean_std_int_buffer( int *buffer, int size,
-	double *pmean, double *pstd );
-int im__find_lroverlap( IMAGE *ref_in, IMAGE *sec_in, IMAGE *out,
-        int bandno_in,
-        int xref, int yref, int xsec, int ysec,
-        int halfcorrelation, int halfarea,
-        int *dx0, int *dy0,
-        double *scale1, double *angle1, double *dx1, double *dy1 );
-int im__find_tboverlap( IMAGE *ref_in, IMAGE *sec_in, IMAGE *out,
-        int bandno_in,
-        int xref, int yref, int xsec, int ysec,
-        int halfcorrelation, int halfarea,
-        int *dx0, int *dy0,
-        double *scale1, double *angle1, double *dx1, double *dy1 );
-int im__find_best_contrast( IMAGE *image,
-	int xpos, int ypos, int xsize, int ysize,
-	int xarray[], int yarray[], int cont[],
-	int nbest, int hcorsize );
-int im__balance( IMAGE *ref, IMAGE *sec, IMAGE *out,
-	IMAGE **ref_out, IMAGE **sec_out, int dx, int dy, int balancetype );
+int im_video_test( IMAGE *im, int brightness, int error );
 
 /* Backwards compatibility macros.
  */
@@ -705,11 +732,15 @@ int im__balance( IMAGE *ref, IMAGE *sec, IMAGE *out,
 
 /* Deprecated API.
  */
-void im_errormsg( const char *fmt, ... );
+void im_errormsg( const char *fmt, ... )
+	__attribute__((format(printf, 1, 2)));
 void im_verrormsg( const char *fmt, va_list ap );
-void im_errormsg_system( int err, const char *fmt, ... );
-void im_diagnostics( const char *fmt, ... );
-void im_warning( const char *fmt, ... );
+void im_errormsg_system( int err, const char *fmt, ... )
+	__attribute__((format(printf, 2, 3)));
+void im_diagnostics( const char *fmt, ... )
+	__attribute__((format(printf, 1, 2)));
+void im_warning( const char *fmt, ... )
+	__attribute__((format(printf, 1, 2)));
 
 #ifdef __cplusplus
 }
