@@ -140,5 +140,5 @@ im_Lab2XYZ_temp( IMAGE *in, IMAGE *out, double X0, double Y0, double Z0 )
 int 
 im_Lab2XYZ( IMAGE *in, IMAGE *out )
 {
-	return( im_Lab2XYZ_temp( in, out, D65_X0, D65_Y0, D65_Z0 ) );
+	return( im_Lab2XYZ_temp( in, out, IM_D65_X0, IM_D65_Y0, IM_D65_Z0 ) );
 }
