@@ -137,7 +137,7 @@ im_bandjoin( IMAGE *in1, IMAGE *in2, IMAGE *out )
 
 	/* Set up the output header.
 	 */
-	if( im_cp_desc( out, in1 ) ) 
+	if( im_cp_descv( out, in1, in2, NULL ) )
                 return( -1 ); 
 	out->Bands = in1->Bands + in2->Bands;
 
