@@ -6,6 +6,14 @@
  *	- some InitializeMagick() fail with NULL arg
  * 2/11/04
  *	- im_magick2vips_header() also checks sensible width/height
+ * 28/10/05
+ * 	- copy attributes to meta
+ * 	- write many-frame images as a big column if all frames have identical
+ * 	  width/height/bands/depth
+ * 31/3/06
+ * 	- test for magick attr support
+ * 8/5/06
+ * 	- set RGB16/GREY16 if appropriate
  */
 
 /*
@@ -46,7 +54,6 @@
 #ifndef HAVE_MAGICK
 
 #include <vips/vips.h>
-#include <vips/thread.h>
 
 int
 im_magick2vips( const char *filename, IMAGE *im )
@@ -71,6 +78,8 @@ im_magick2vips_header( const char *filename, IMAGE *im )
 #include <sys/types.h>
 
 #include <vips/vips.h>
+#include <vips/vbuf.h>
+#include <vips/thread.h>
 
 #include <magick/api.h>
 
@@ -81,13 +90,17 @@ im_magick2vips_header( const char *filename, IMAGE *im )
 /* What we track during a read call.
  */
 typedef struct _Read {
-	const char *filename;
+	char *filename;
 	IMAGE *im;
 
 	Image *image;
 	ImageInfo *image_info;
 	ExceptionInfo exception;
 
+	int n_frames;
+	Image **frames;
+	int frame_height;
+
 	/* Mutex to serialise calls to libMagick during threaded read.
 	 */
 	GMutex *lock;
@@ -100,18 +113,10 @@ read_destroy( Read *read )
 	printf( "im_magick2vips: read_destroy: %s\n", read->filename );
 #endif /*DEBUG*/
 
-	if( read->image ) {
-		DestroyImage( read->image );
-		read->image = NULL;
-	}
-	if( read->image_info ) {
-		DestroyImageInfo( read->image_info );
-		read->image_info = NULL;
-	}
-	if( read->filename ) {
-		im_free( (void *) read->filename );
-		read->filename = NULL;
-	}
+	IM_FREEF( DestroyImage, read->image );
+	IM_FREEF( DestroyImageInfo, read->image_info ); 
+	IM_FREE( read->frames );
+	IM_FREE( read->filename );
 	DestroyExceptionInfo( &read->exception );
 	IM_FREEF( g_mutex_free, read->lock );
 	im_free( read );
@@ -137,6 +142,9 @@ read_new( const char *filename, IMAGE *im )
 	read->image = NULL;
 	read->image_info = CloneImageInfo( NULL );
 	GetExceptionInfo( &read->exception );
+	read->n_frames = 0;
+	read->frames = NULL;
+	read->frame_height = 0;
 	read->lock = g_mutex_new();
 
         if( im_add_close_callback( im,
@@ -158,32 +166,25 @@ read_new( const char *filename, IMAGE *im )
 }
 
 static int
-parse_header( Image *image, IMAGE *im )
+get_bands( Image *image )
 {
-	char hist[MaxTextExtent];
-
-	im->Xsize = image->columns;
-	im->Ysize = image->rows;
+	int bands;
 
 	switch( image->colorspace ) {
 	case GRAYColorspace:
-		im->Bands = 1;
-		im->Type = IM_TYPE_B_W;
+		bands = 1;
 		break;
 
 	case RGBColorspace:
-		im->Bands = 3;
-		im->Type = IM_TYPE_RGB;
+		bands = 3;
 		break;
 
 	case sRGBColorspace:
-		im->Bands = 3;
-		im->Type = IM_TYPE_RGB;
+		bands = 3;
 		break;
 
 	case CMYKColorspace:
-		im->Bands = 4;
-		im->Type = IM_TYPE_CMYK;
+		bands = 4;
 		break;
 
 	default:
@@ -196,9 +197,28 @@ parse_header( Image *image, IMAGE *im )
 	 */
 	if( image->matte ) {
 		assert( image->colorspace != CMYKColorspace );
-		im->Bands += 1;
+		bands += 1;
 	}
 
+	return( bands );
+}
+
+static int
+parse_header( Read *read )
+{
+	IMAGE *im = read->im;
+	Image *image = read->image;
+
+	const ImageAttribute *attr;
+	Image *p;
+	int i;
+
+	im->Xsize = image->columns;
+	im->Ysize = image->rows;
+	read->frame_height = image->rows;
+	if( (im->Bands = get_bands( image )) < 0 )
+		return( -1 );
+
 	switch( image->depth ) {
 	case 8:
 		im->BandFmt = IM_BANDFMT_UCHAR;
@@ -216,6 +236,38 @@ parse_header( Image *image, IMAGE *im )
 		return( -1 );
 	}
 
+	switch( image->colorspace ) {
+	case GRAYColorspace:
+		if( im->BandFmt == IM_BANDFMT_USHORT )
+			im->Type = IM_TYPE_GREY16;
+		else
+			im->Type = IM_TYPE_B_W;
+		break;
+
+	case RGBColorspace:
+		if( im->BandFmt == IM_BANDFMT_USHORT )
+			im->Type = IM_TYPE_RGB16;
+		else
+			im->Type = IM_TYPE_RGB;
+		break;
+
+	case sRGBColorspace:
+		if( im->BandFmt == IM_BANDFMT_USHORT )
+			im->Type = IM_TYPE_RGB16;
+		else
+			im->Type = IM_TYPE_sRGB;
+		break;
+
+	case CMYKColorspace:
+		im->Type = IM_TYPE_CMYK;
+		break;
+
+	default:
+		im_error( "im_magick2vips", _( "unsupported colorspace %d" ),
+			(int) image->colorspace );
+		return( -1 );
+	}
+
 	switch( image->units ) {
 	case PixelsPerInchResolution:
 		im->Xres = image->x_resolution / 25.4;
@@ -236,16 +288,63 @@ parse_header( Image *image, IMAGE *im )
 	/* Other fields.
 	 */
 	im->Coding = IM_CODING_NONE;
-	im_snprintf( hist, MaxTextExtent, "%s image imported by libMagick\n",
-		image->magick );
-	im->Hist = im_strdup( NULL, hist );
+
+#ifdef HAVE_GETNEXTIMAGEATTRIBUTE
+	/* Gah, magick6.something and later only. Attach any attributes.
+	 */
+	ResetImageAttributeIterator( image );
+	while( (attr = GetNextImageAttribute( image )) ) {
+		char name_text[256];
+		VBuf name;
+
+		im_buf_init_static( &name, name_text, 256 );
+		im_buf_appendf( &name, "magick-%s", attr->key );
+		im_meta_set_string( im, im_buf_all( &name ), attr->value );
+
+#ifdef DEBUG
+		printf( "key = \"%s\", value = \"%s\"\n", 
+			attr->key, attr->value );
+#endif /*DEBUG*/
+	}
+#endif /*HAVE_GETNEXTIMAGEATTRIBUTE*/
+
+	/* Do we have a set of equal-sized frames? Append them.
+
+	   	FIXME ... there must be an attribute somewhere from dicom read 
+		which says this is a volumetric image
+
+	 */
+	read->n_frames = 0;
+	for( p = image; p; (p = GetNextImageInList( p )) ) {
+		if( p->columns != im->Xsize ||
+			p->rows != im->Ysize ||
+			p->depth != im->Bbits ||
+			get_bands( p ) != im->Bands )
+			break;
+
+		read->n_frames += 1;
+	}
+	if( p ) 
+		/* Nope ... just do the first image in the list.
+		 */
+		read->n_frames = 1;
+
+	/* Record frame pointers.
+	 */
+	im->Ysize *= read->n_frames;
+	if( !(read->frames = IM_ARRAY( NULL, read->n_frames, Image * )) )
+		return( -1 );
+	p = image;
+	for( i = 0; i < read->n_frames; i++ ) {
+		read->frames[i] = p;
+		p = GetNextImageInList( p );
+	}
 
 	return( 0 );
 }
 
 static void
-unpack_pixels( IMAGE *im, 
-	unsigned char *q8, PixelPacket *pixels, int n )
+unpack_pixels( IMAGE *im, PEL *q8, PixelPacket *pixels, int n )
 {
 	unsigned short *q16 = (unsigned short *) q8;
 	int x;
@@ -397,11 +496,15 @@ magick_fill_region( REGION *out, void *dummy, Read *read )
 	int y;
 
 	for( y = 0; y < r->height; y++ ) {
+		int top = r->top + y;
+		int frame = top / read->frame_height;
+		int line = top % read->frame_height;
+
 		PixelPacket *pixels;
 
 		g_mutex_lock( read->lock );
-		pixels = get_pixels( read->image, 
-			r->left, r->top + y, r->width, 1 );
+		pixels = get_pixels( read->frames[frame], 
+			r->left, line, r->width, 1 );
 		g_mutex_unlock( read->lock );
 
 		if( !pixels ) {
@@ -411,7 +514,7 @@ magick_fill_region( REGION *out, void *dummy, Read *read )
 		}
 
 		unpack_pixels( read->im, 
-			IM_REGION_ADDR( out, r->left, r->top + y ), 
+			(PEL *) IM_REGION_ADDR( out, r->left, top ), 
 			pixels, r->width );
 	}
 
@@ -435,7 +538,7 @@ im_magick2vips( const char *filename, IMAGE *im )
 		return( -1 );
 	}
 
-	if( parse_header( read->image, im ) ||
+	if( parse_header( read ) ||
 		im_poutcheck( im ) || 
 		im_generate( im, NULL, magick_fill_region, NULL, read, NULL ) )
 		return( -1 );
@@ -460,7 +563,7 @@ im_magick2vips_header( const char *filename, IMAGE *im )
 		return( -1 );
 	}
 
-	if( parse_header( read->image, im ) ) 
+	if( parse_header( read ) ) 
 		return( -1 );
 
 	if( im->Xsize <= 0 || im->Ysize <= 0 ) {
