@@ -82,8 +82,15 @@
  *	- also LABS images 
  * 22/6/05
  *	- 16 bit LAB write was broken
+ * 9/9/05
+ * 	- write any icc profile from meta 
  * 3/3/06
  * 	- raise tile buffer limit (thanks Ruven)
+ * 11/11/06
+ * 	- set ORIENTATION_TOPLEFT (thanks Josef)
+ * 18/7/07 Andrey Kiselev
+ * 	- remove "b" option on TIFFOpen()
+ * 	- support TIFFTAG_PREDICTOR types for lzw and deflate compression
  */
 
 /*
@@ -213,6 +220,7 @@ typedef struct tiff_write {
 
 	int compression;		/* Compression type */
 	int jpqual;			/* JPEG q-factor */
+	int predictor;			/* Predictor value */
 	int tile;			/* Tile or not */
 	int tilew, tileh;		/* Tile size */
 	int pyramid;			/* Write pyramid */
@@ -240,11 +248,7 @@ tiff_openout( char *name )
 {
 	TIFF *tif;
 
-#ifdef BINARY_OPEN
-	if( !(tif = TIFFOpen( name, "wb" )) ) {
-#else /*BINARY_OPEN*/
 	if( !(tif = TIFFOpen( name, "w" )) ) {
-#endif /*BINARY_OPEN*/
 		im_error( "im_vips2tiff", 
 			_( "unable to open \"%s\" for output" ), name );
 		return( NULL );
@@ -260,11 +264,7 @@ tiff_openin( char *name )
 {
 	TIFF *tif;
 
-#ifdef BINARY_OPEN
-	if( !(tif = TIFFOpen( name, "rb" )) ) {
-#else /*BINARY_OPEN*/
 	if( !(tif = TIFFOpen( name, "r" )) ) {
-#endif /*BINARY_OPEN*/
 		im_error( "im_vips2tiff", 
 			_( "unable to open \"%s\" for input" ), name );
 		return( NULL );
@@ -324,8 +324,8 @@ static void
 LabS2Lab16( PEL *q, PEL *p, int n )
 {
         int x;
-	short *p1 = (unsigned short *) p;
-	unsigned short *q1 = (short *) q;
+	short *p1 = (short *) p;
+	unsigned short *q1 = (unsigned short *) q;
 
         for( x = 0; x < n; x++ ) {
                 /* TIFF uses unsigned 16 bit ... move zero, scale up L.
@@ -370,7 +370,7 @@ static int
 embed_profile( TIFF *tif, const char *profile )
 {
 	char *buffer;
-	int length;
+	unsigned int length;
 
 	if( !(buffer = im__file_read_name( profile, &length )) ) 
 		return( -1 );
@@ -384,6 +384,25 @@ embed_profile( TIFF *tif, const char *profile )
 	return( 0 );
 }
 
+/* Embed an ICC profile from IMAGE metadata.
+ */
+static int
+embed_profile_meta( TIFF *tif, IMAGE *im )
+{
+	void *data;
+	size_t data_length;
+
+	if( im_meta_get_blob( im, IM_META_ICC_NAME, &data, &data_length ) )
+		return( -1 );
+	TIFFSetField( tif, TIFFTAG_ICCPROFILE, data_length, data );
+
+#ifdef DEBUG
+	printf( "im_vips2tiff: attached profile from meta\n" );
+#endif /*DEBUG*/
+
+	return( 0 );
+}
+
 /* Write a TIFF header. width and height are the size of the IMAGE we are
  * writing (may have been shrunk!).
  */
@@ -397,6 +416,7 @@ write_tiff_header( TiffWrite *tw, TIFF *tif, int width, int height )
 	TIFFSetField( tif, TIFFTAG_IMAGEWIDTH, width );
 	TIFFSetField( tif, TIFFTAG_IMAGELENGTH, height );
 	TIFFSetField( tif, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG );
+	TIFFSetField( tif, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT );
 	TIFFSetField( tif, TIFFTAG_COMPRESSION, tw->compression );
 
 	/* Don't write mad resolutions (eg. zero), it confuses some programs.
@@ -412,10 +432,16 @@ write_tiff_header( TiffWrite *tw, TIFF *tif, int width, int height )
 		TIFFSetField( tif, TIFFTAG_JPEGCOLORMODE, JPEGCOLORMODE_RGB );
 	}
 
+	if( tw->predictor != -1 ) 
+		TIFFSetField( tif, TIFFTAG_PREDICTOR, tw->predictor );
+
 	/* Attach ICC profile.
 	 */
 	if( tw->embed && embed_profile( tif, tw->icc_profile ) )
 		return( -1 );
+	if( !tw->embed && im_header_get_type( tw->im, IM_META_ICC_NAME ) &&
+		embed_profile_meta( tif, tw->im ) )
+		return( -1 );
 
 	/* And colour fields.
 	 */
@@ -666,7 +692,7 @@ find_tile( PyramidLayer *layer, Rect *pos )
 	 */
 	if( (i = find_new_tile( layer )) < 0 )
 		return( -1 );
-	if( im_region_local( layer->tiles[i].tile, pos ) )
+	if( im_region_buffer( layer->tiles[i].tile, pos ) )
 		return( -1 );
 	layer->tiles[i].bits = PYR_NONE;
 
@@ -891,8 +917,8 @@ new_tile( PyramidLayer *layer, REGION *tile, Rect *area )
 	 */
 	new.left = area->left / 2;
 	new.top = area->top / 2;
-	new.width = area->width / 2;;
-	new.height = area->height / 2;;
+	new.width = area->width / 2;
+	new.height = area->height / 2;
 
 	/* Has size fallen to zero? Can happen if this is a one-pixel-wide
 	 * strip.
@@ -1151,6 +1177,7 @@ make_tiff_write( IMAGE *im, const char *filename )
 	tw->tbuf = NULL;
 	tw->compression = COMPRESSION_NONE;
 	tw->jpqual = 75;
+	tw->predictor = -1;
 	tw->tile = 0;
 	tw->tilew = 128;
 	tw->tileh = 128;
@@ -1176,10 +1203,28 @@ make_tiff_write( IMAGE *im, const char *filename )
 			tw->compression = COMPRESSION_PACKBITS;
 		else if( im_isprefix( "ccittfax4", q ) ) 
 			tw->compression = COMPRESSION_CCITTFAX4;
-		else if( im_isprefix( "lzw", q ) ) 
+		else if( im_isprefix( "lzw", q ) ) {
 			tw->compression = COMPRESSION_LZW;
-		else if( im_isprefix( "deflate", q ) ) 
-			tw->compression = COMPRESSION_DEFLATE;
+
+			if( (r = im_getsuboption( q )) ) 
+				if( sscanf( r, "%d", &tw->predictor ) != 1 ) {
+					im_error( "im_vips2tiff",
+						_( "bad predictor "
+							"parameter" ) );
+					return( NULL );
+				}
+		}
+		else if( im_isprefix( "deflate", q ) ) {
+			tw->compression = COMPRESSION_ADOBE_DEFLATE;
+
+			if( (r = im_getsuboption( q )) ) 
+				if( sscanf( r, "%d", &tw->predictor ) != 1 ) {
+					im_error( "im_vips2tiff",
+						_( "bad predictor "
+							"parameter" ) );
+					return( NULL );
+				}
+		}
 		else if( im_isprefix( "jpeg", q ) ) {
 			tw->compression = COMPRESSION_JPEG;
 
@@ -1369,11 +1414,13 @@ tiff_copy( TIFF *out, TIFF *in )
 	CopyField( TIFFTAG_IMAGEWIDTH, i32 );
 	CopyField( TIFFTAG_IMAGELENGTH, i32 );
 	CopyField( TIFFTAG_PLANARCONFIG, i16 );
+	CopyField( TIFFTAG_ORIENTATION, i16 );
 	CopyField( TIFFTAG_COMPRESSION, i16 );
 	CopyField( TIFFTAG_XRESOLUTION, f );
 	CopyField( TIFFTAG_YRESOLUTION, f );
 	CopyField( TIFFTAG_RESOLUTIONUNIT, i16 );
 	CopyField( TIFFTAG_JPEGQUALITY, i );
+        CopyField( TIFFTAG_PREDICTOR, i16 );
 	CopyField( TIFFTAG_SAMPLESPERPIXEL, i16 );
 	CopyField( TIFFTAG_BITSPERSAMPLE, i16 );
 	CopyField( TIFFTAG_PHOTOMETRIC, i16 );
