@@ -58,21 +58,6 @@ im_image( void *buffer, int width, int height, int bands, int format )
 {
 	IMAGE *im;
 
-	/* Values for bbits, index by bandfmt.
-	 */
-	static int bits[10] = {
-		IM_BBITS_BYTE,		/* IM_BANDFMT_UCHAR	*/
-		IM_BBITS_BYTE,		/* IM_BANDFMT_CHAR	*/
-		IM_BBITS_SHORT,		/* IM_BANDFMT_USHORT	*/
-		IM_BBITS_SHORT,		/* IM_BANDFMT_SHORT	*/
-		IM_BBITS_INT,		/* IM_BANDFMT_UINT	*/
-		IM_BBITS_INT,		/* IM_BANDFMT_INT	*/
-		IM_BBITS_FLOAT,		/* IM_BANDFMT_FLOAT	*/
-		IM_BBITS_COMPLEX,	/* IM_BANDFMT_COMPLEX	*/
-		IM_BBITS_DOUBLE,	/* IM_BANDFMT_DOUBLE	*/
-		IM_BBITS_DPCOMPLEX	/* IM_BANDFMT_DPCOMPLEX	*/
-	};
-
 	if( width <= 0 || height <= 0 || bands <= 0 || 
 		format < 0 || format > IM_BANDFMT_DPCOMPLEX ) {
 		im_errormsg( "im_image: bad parameters" );
@@ -90,7 +75,7 @@ im_image( void *buffer, int width, int height, int bands, int format )
 	im->Ysize = height;
 	im->Bands = bands;
 	im->BandFmt = format;
-	im->Bbits = bits[format];
+	im->Bbits = im_bits_of_fmt( format );
 	im->Coding = IM_CODING_NONE;
 
 	if( bands == 1 )
