@@ -85,17 +85,17 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif /*HAVE_UNISTD_H*/
-#ifdef HAVE_WINDOWS_H 
+#ifdef OS_WIN32 
 #ifndef S_ISREG
 #define S_ISREG(m) (!!(m & _S_IFREG))
 #endif
-#endif /*HAVE_WINDOWS_H*/
+#endif /*OS_WIN32*/
 
 #include <vips/vips.h>
 
-#ifdef HAVE_WINDOWS_H
+#ifdef OS_WIN32
 #include <windows.h>
-#endif /*HAVE_WINDOWS_H*/
+#endif /*OS_WIN32*/
 
 #ifdef WITH_DMALLOC
 #include <dmalloc.h>
@@ -110,7 +110,7 @@ im__mmap( int fd, int writeable, size_t length, gint64 offset )
 	printf( "im__mmap: length = %d, offset = %lld\n", length, offset );
 #endif /*DEBUG*/
 
-#ifdef HAVE_WINDOWS_H
+#ifdef OS_WIN32
 {
 	HANDLE hFile = (HANDLE) _get_osfhandle( fd );
         HANDLE hMMFile;
@@ -159,7 +159,7 @@ im__mmap( int fd, int writeable, size_t length, gint64 offset )
 	 */
 	CloseHandle( hMMFile );
 }
-#else /*!HAVE_WINDOWS_H*/
+#else /*!OS_WIN32*/
 {
 	int prot;
 
@@ -181,7 +181,7 @@ im__mmap( int fd, int writeable, size_t length, gint64 offset )
 		return( NULL ); 
 	}
 }
-#endif /*HAVE_WINDOWS_H*/
+#endif /*OS_WIN32*/
 
 	return( baseaddr );
 }
@@ -189,19 +189,19 @@ im__mmap( int fd, int writeable, size_t length, gint64 offset )
 int
 im__munmap( void *start, size_t length )
 {
-#ifdef HAVE_WINDOWS_H
+#ifdef OS_WIN32
 	if( !UnmapViewOfFile( start ) ) {
 		im_error_system( GetLastError(), "im_mapfile",
 			_( "unable to UnmapViewOfFile" ) );
 		return( -1 );
 	}
-#else /*!HAVE_WINDOWS_H*/
+#else /*!OS_WIN32*/
 	if( munmap( start, length ) < 0 ) {
 		im_error_system( errno, "im_mapfile", 
 			_( "unable to munmap file" ) );
 		return( -1 );
 	}
-#endif /*HAVE_WINDOWS_H*/
+#endif /*OS_WIN32*/
 
 	return( 0 );
 }
@@ -290,7 +290,7 @@ im_remapfilerw( IMAGE *image )
 {
 	void *baseaddr;
 
-#ifdef HAVE_WINDOWS_H
+#ifdef OS_WIN32
 {
 	HANDLE hFile = (HANDLE) _get_osfhandle( image->fd );
         HANDLE hMMFile;
@@ -322,7 +322,7 @@ im_remapfilerw( IMAGE *image )
 	 */
 	CloseHandle( hMMFile );
 }
-#else /*!HAVE_WINDOWS_H*/
+#else /*!OS_WIN32*/
 {
 	assert( image->dtype == IM_MMAPIN );
 
@@ -335,7 +335,7 @@ im_remapfilerw( IMAGE *image )
 		return( -1 ); 
 	}
 }
-#endif /*HAVE_WINDOWS_H*/
+#endif /*OS_WIN32*/
 
 	image->dtype = IM_MMAPINRW;
 
