@@ -26,6 +26,8 @@
  * 1/4/05
  *	- rewritten for a few global threads instead, and a job queue ...
  *	  should be simpler & more reliable
+ * 23/4/07
+ * 	- oop, race condition fixed
  */
 
 /*
@@ -56,10 +58,10 @@
 
 /* Turn on debugging output.
 #define DEBUG
-#define DEBUG_PAINT
 #define DEBUG_REUSE
 #define DEBUG_TG
 #define DEBUG_MAKE
+#define DEBUG_PAINT
  */
 
 #ifdef HAVE_CONFIG_H
@@ -247,7 +249,7 @@ static int
 render_free( Render *render )
 {
 #ifdef DEBUG_MAKE
-	printf( "render_free: 0x%x\n", (unsigned int) render );
+	printf( "render_free: %p\n", render );
 #endif /*DEBUG_MAKE*/
 
 	assert( render->ref_count == 0 );
@@ -361,7 +363,8 @@ render_dirty_process( Render *render )
 	/* Take a tile off the dirty list.
 	 */
 	g_mutex_lock( render->dirty_lock );
-	if( render->dirty ) { tile = (Tile *) render->dirty->data;
+	if( render->dirty ) { 
+		tile = (Tile *) render->dirty->data;
 		assert( tile->state == TILE_DIRTY );
 		render->dirty = g_slist_remove( render->dirty, tile );
 		tile->state = TILE_WORKING;
@@ -379,19 +382,24 @@ render_dirty_process( Render *render )
 
 #ifdef DEBUG_TG
 			printf( "render_paint_tile: "
-				"0x%x starting threadgroup\n",
-				(unsigned int) render );
+				"%p starting threadgroup\n",
+				render );
 #endif /*DEBUG_TG*/
 		}
 
 		if( render->tg ) {
 #ifdef DEBUG_PAINT
 			printf( "render_fill_tile: "
-				"0x%x paint of tile %dx%d\n",
-				(unsigned int) render,
+				"%p paint of tile %dx%d\n",
+				render,
 				tile->area.left, tile->area.top );
 #endif /*DEBUG_PAINT*/
 
+			/* We're writing to the tile region, but we didn't 
+			 * make it.
+			 */
+			im__region_take_ownership( tile->region );
+
 			result = im_prepare_thread( render->tg, 
 				tile->region, &tile->area );
 		}
@@ -409,9 +417,15 @@ render_dirty_process( Render *render )
 #endif /*DEBUG_PAINT*/
 
 		g_mutex_lock( render->fade_lock );
+
 		render->fade = g_slist_prepend( render->fade, tile );
 		tile->time = render->time;
 		tile->state = TILE_PAINTED_FADING;
+
+		/* Hand tile over to another thread.
+		 */
+		im__region_no_ownership( tile->region );
+
 		g_mutex_unlock( render->fade_lock );
 	}
 }
@@ -442,8 +456,7 @@ render_dirty_put( Render *render )
 		/* Coming off the jobs list ... shut down the render pipeline.
 		 */
 #ifdef DEBUG_TG
-		printf( "render_dirty_put: 0x%x stopping threadgroup\n",
-			(unsigned int) render );
+		printf( "render_dirty_put: %p stopping threadgroup\n", render );
 #endif /*DEBUG_TG*/
 		IM_FREEF( im_threadgroup_free, render->tg );
 	}
@@ -592,7 +605,7 @@ render_fade( void *client )
 
 			/* Ask client to update.
 			 */
-			if( render->notify )
+			if( render->notify ) 
 				render->notify( render->out, 
 					&tile->area, render->client );
 		}
@@ -771,10 +784,18 @@ tile_set_dirty( Tile *tile, Rect *area )
 	render->access_ticks += 1;
 
 	g_mutex_lock( render->dirty_lock );
+
 	tile->state = TILE_DIRTY;
 	tile->area = *area;
 	render->dirty = g_slist_prepend( render->dirty, tile );
-	g_mutex_unlock( render->dirty_lock );
+
+	/* Someone else will write to it now.
+	 */
+	im__region_no_ownership( tile->region );
+
+	/* Can't unlock render->dirty_lock here, we need to render_dirty_put()
+	 * before the tile is processed.
+	 */
 
 	if( render->notify && have_threads ) 
 		/* Add to the list of renders with dirty tiles. One of our bg 
@@ -793,6 +814,11 @@ tile_set_dirty( Tile *tile, Rect *area )
 
 		if( !render->tg )
 			render->tg = im_threadgroup_create( render->in );
+
+		/* We're writing to the tile region, but we didn't make it.
+		 */
+		im__region_take_ownership( tile->region );
+
 		if( render->tg )
 			im_prepare_thread( render->tg, 
 				tile->region, &tile->area );
@@ -802,6 +828,8 @@ tile_set_dirty( Tile *tile, Rect *area )
 		tile->state = TILE_PAINTED;
 		render->dirty = g_slist_remove( render->dirty, tile );
 	}
+
+	g_mutex_unlock( render->dirty_lock );
 }
 
 static void *
@@ -829,8 +857,7 @@ render_tile_get_painted( Render *render )
 		assert( tile->state == TILE_PAINTED );
 
 #ifdef DEBUG_REUSE
-		printf( "render_tile_get_painted: reusing painted 0x%x\n",
-			(unsigned int) tile );
+		printf( "render_tile_get_painted: reusing painted %p\n", tile );
 
 		g_mutex_lock( render->dirty_lock );
 		assert( !g_slist_find( render->dirty, tile ) );
@@ -866,8 +893,7 @@ render_tile_get_dirty( Render *render )
 
 #ifdef DEBUG_REUSE
 	if( tile )
-		printf( "render_tile_get_dirty: reusing dirty 0x%x\n",
-			(unsigned int) tile );
+		printf( "render_tile_get_dirty: reusing dirty %p\n", tile );
 #endif /*DEBUG_REUSE*/
 
 	return( tile );
@@ -880,35 +906,45 @@ render_tile_get( Render *render, Rect *area )
 
 	/* Got this tile already?
 	 */
-	if( (tile = render_tile_lookup( render, area )) ) 
+	if( (tile = render_tile_lookup( render, area )) ) {
+#ifdef DEBUG_PAINT
+		printf( "render_tile_get: found %dx%d in cache\n",
+			area->left, area->top );
+#endif /*DEBUG_PAINT*/
+
 		return( tile );
+	}
 
 	/* Have we fewer tiles than teh max? Can just make a new tile.
 	 */
 	if( render->ntiles < render->max || render->max == -1 ) {
 		if( !(tile = tile_new( render )) ) 
 			return( NULL );
-
-		tile_set_dirty( tile, area );
-		return( tile );
 	}
+	else {
+		/* Need to reuse a tile. Try for an old painted tile first, 
+		 * then if that fails, reuse a dirty tile. Don't search the 
+		 * fading list, though we could maybe scavenge there.
+		 */
+		if( !(tile = render_tile_get_painted( render )) &&
+			!(tile = render_tile_get_dirty( render )) ) 
+			return( NULL );
 
-	/* Need to reuse a tile. Try for an old painted tile first, then if
-	 * that fails, reuse a dirty tile. Don't search the fading list,
-	 * though we could maybe scavenge there.
-	 */
-	if( (tile = render_tile_get_painted( render )) ||
-		(tile = render_tile_get_dirty( render )) ) {
 #ifdef DEBUG_REUSE
 		printf( "(render_tile_get: was at %dx%d, moving to %dx%d)\n",
 			tile->area.left, tile->area.top,
 			area->left, area->top );
 #endif /*DEBUG_REUSE*/
-		tile_set_dirty( tile, area );
-		return( tile );
 	}
 
-	return( NULL );
+#ifdef DEBUG_PAINT
+	printf( "render_tile_get: sending %dx%d for calc\n",
+		area->left, area->top );
+#endif /*DEBUG_PAINT*/
+
+	tile_set_dirty( tile, area );
+
+	return( tile );
 }
 
 /* Copy what we can from the tile into the region.
@@ -929,7 +965,13 @@ tile_copy( Tile *tile, REGION *to )
 	/* If the tile is painted, copy over the pixels. Otherwise, fill with
 	 * zero. 
 	 */
-	if( tile->state == TILE_PAINTED || tile->state == TILE_PAINTED_FADING ) 
+	if( tile->state == TILE_PAINTED || 
+		tile->state == TILE_PAINTED_FADING ) {
+#ifdef DEBUG_PAINT
+		printf( "tile_copy: copying calculated pixels for %dx%d\n",
+			tile->area.left, tile->area.top ); 
+#endif /*DEBUG_PAINT*/
+
 		for( y = ovlap.top; y < IM_RECT_BOTTOM( &ovlap ); y++ ) {
 			PEL *p = (PEL *) IM_REGION_ADDR( tile->region, 
 				ovlap.left, y );
@@ -937,7 +979,13 @@ tile_copy( Tile *tile, REGION *to )
 
 			memcpy( q, p, len );
 		}
+	}
 	else {
+#ifdef DEBUG_PAINT
+		printf( "tile_copy: zero filling for %dx%d\n",
+			tile->area.left, tile->area.top ); 
+#endif /*DEBUG_PAINT*/
+
 		for( y = ovlap.top; 
 			y < IM_RECT_BOTTOM( &ovlap ); y++ ) {
 			PEL *q = (PEL *) 
@@ -1094,18 +1142,22 @@ im_render_fade( IMAGE *in, IMAGE *out, IMAGE *mask,
 	}
 	if( im_cp_desc( out, in ) )
 		return( -1 );
+	if( im_demand_hint( out, IM_SMALLTILE, in, NULL ) ||
+		im_demand_hint( mask, IM_SMALLTILE, in, NULL ) )
+		return( -1 );
 
 	if( !(render = render_new( in, out, mask, 
 		width, height, max, fps, steps, priority, notify, client )) )
 		return( -1 );
 
 #ifdef DEBUG_MAKE
-	printf( "im_render: max = %d, 0x%x\n", max, (unsigned int) render );
+	printf( "im_render: max = %d, %p\n", max, render );
 #endif /*DEBUG_MAKE*/
 
 	if( im_generate( out, NULL, region_fill, NULL, render, NULL ) )
 		return( -1 );
-	if( mask && im_generate( mask, NULL, mask_fill, NULL, render, NULL ) )
+	if( mask && 
+		im_generate( mask, NULL, mask_fill, NULL, render, NULL ) )
 		return( -1 );
 
 	return( 0 );
@@ -1122,10 +1174,7 @@ im_render( IMAGE *in, IMAGE *out, IMAGE *mask,
 int 
 im_cache( IMAGE *in, IMAGE *out, int width, int height, int max )
 {
-	IMAGE *t;
-
-	if( !(t = im_open_local( out, "im_cache", "p" )) ||
-		im_render( in, out, t, width, height, max, NULL, NULL ) )
+	if( im_render( in, out, NULL, width, height, max, NULL, NULL ) )
 		return( -1 );
 
 	return( 0 );
