@@ -20,8 +20,8 @@
  *	- HAVE_TIFF turns on TIFFness
  * 19/10/02 HB
  *      - PNG added
- * 5/1/02 JC
- *	- DICOM added
+ * 1/5/06
+ * 	- added exr
  */
 
 /*
@@ -189,6 +189,31 @@ im_isfloat( IMAGE *im )
 	}
 }
 
+int
+im_isscalar( IMAGE *im )
+{	
+	switch( im->BandFmt ) {
+	case IM_BANDFMT_UCHAR:
+	case IM_BANDFMT_CHAR:
+	case IM_BANDFMT_USHORT:
+	case IM_BANDFMT_SHORT:
+	case IM_BANDFMT_UINT:
+	case IM_BANDFMT_INT:
+	case IM_BANDFMT_FLOAT:
+	case IM_BANDFMT_DOUBLE:	
+		return( 1 );
+
+	case IM_BANDFMT_COMPLEX:
+	case IM_BANDFMT_DPCOMPLEX:	
+		return( 0 );
+	
+	default:
+		error_exit( "im_isscalar: unknown image BandFmt" );
+		/*NOTREACHED*/
+		return( -1 );
+	}
+}
+
 int
 im_iscomplex( IMAGE *im )
 {	
@@ -434,6 +459,32 @@ im_isvips( const char *filename )
 		return( 0 );
 }
 
+int
+im_isexr( const char *filename )
+{
+	unsigned char buf[4];
+	int fd;
+
+#ifdef BINARY_OPEN
+	if( (fd = open( filename, O_RDONLY | O_BINARY )) == -1 )
+#else /*BINARY_OPEN*/
+	if( (fd = open( filename, O_RDONLY )) == -1 )
+#endif /*BINARY_OPEN*/
+		return( 0 );
+	if( read( fd, buf, 4 ) != 4 ) {
+		im_errormsg( "im_isexr: read failed on %s", filename );
+		close( fd );
+		return( 0 );
+	}
+	close( fd );
+
+	if( buf[0] == 0x76 && buf[1] == 0x2f &&
+                buf[2] == 0x31 && buf[3] == 0x01 )
+		return( 1 );
+
+	return( 0 );
+}
+
 /* Test for file exists.
  */
 int
@@ -515,7 +566,7 @@ im_ispoweroftwo( int p )
 {
 	int i, n;
 
-	/* Count set bits.
+	/* Count set bits. Could use a LUT, I guess.
 	 */
 	for( i = 0, n = 0; p; i++, p >>= 1 )
 		if( (p & 1) == 1 )
