@@ -51,6 +51,8 @@
  *	  against geo reconstruct errors
  * 10/3/03 JC
  *	- weed out overlaps which contain only transparent pixels
+ * 4/1/07
+ * 	- switch to new history thing, switch im_errormsg() too
  */
 
 /*
@@ -141,7 +143,7 @@ break_items( char *line, char **out )
 		out[i] = line = p + 1;
 
 		if( !(p = strchr( line, '>' )) ) {
-			im_errormsg( "break_files: no matching '>'" );
+			im_error( "break_files", _( "no matching '>'" ) );
 			return( -1 );
 		}
 
@@ -150,7 +152,7 @@ break_items( char *line, char **out )
 	}
 
 	if( i == MAX_ITEMS ) {
-		im_errormsg( "break_files: too many items" );
+		im_error( "break_files", _( "too many items" ) );
 		return( -1 );
 	}
 
@@ -445,7 +447,7 @@ propogate_transform( JoinNode *node, Transformation *trn )
 		return( 0 );
 
 	if( node->dirty && node->arg1 && node->arg2 ) {
-		im_errormsg( "im_global_balance: circularity detected" );
+		im_error( "im_global_balance", _( "circularity detected" ) );
 		return( -1 );
 	}
 	node->dirty = 1;
@@ -478,8 +480,8 @@ make_join( SymbolTable *st, JoinType type,
 	/* Check output is ok.
 	 */
 	if( out->type != JOIN_LEAF ) {
-		im_errormsg( "im_global_balance: image \"%s\" used twice "
-			"as output", out->name );
+		im_error( "im_global_balance", 
+			_( "image \"%s\" used twice as output" ), out->name );
 		return( -1 );
 	}
 
@@ -534,8 +536,8 @@ make_copy( SymbolTable *st, JoinNode *before, JoinNode *after )
 	/* Check output is ok.
 	 */
 	if( after->type != JOIN_LEAF ) {
-		im_errormsg( "im_global_balance: image \"%s\" used twice "
-			"as output", after->name );
+		im_error( "im_global_balance", 
+			_( "image \"%s\" used twice as output" ), after->name );
 		return( -1 );
 	}
 
@@ -555,12 +557,18 @@ make_copy( SymbolTable *st, JoinNode *before, JoinNode *after )
 /* Process a single .desc line.
  */
 static int
-process_line( SymbolTable *st, char *line )
+process_line( SymbolTable *st, const char *text )
 {
+	char line[1024];
+
 #ifdef DEBUG
-	printf( "read: %s\n", line );
+	printf( "read: %s\n", text );
 #endif /*DEBUG*/
 
+	/* We destroy line during the parse.
+	 */
+	im_strncpy( line, text, 1024 );
+
 	if( im_isprefix( "#LRJOIN ", line ) || 
 		im_isprefix( "#TBJOIN ", line ) ) {
 		/* Yes: magic join command. Break into tokens. Format is eg.
@@ -577,8 +585,8 @@ process_line( SymbolTable *st, char *line )
 		if( (nitems = break_items( line, item )) < 0 )
 			return( -1 );
 		if( nitems != 5 && nitems != 6 ) {
-			im_errormsg( "global_balance: bad number of args "
-				"in join line" );
+			im_error( "global_balance", 
+				_( "bad number of args in join line" ) );
 			return( -1 );
 		}
 
@@ -619,8 +627,8 @@ process_line( SymbolTable *st, char *line )
 		if( (nitems = break_items( line, item )) < 0 )
 			return( -1 );
 		if( nitems != 7 && nitems != 8 ) {
-			im_errormsg( "global_balance: bad number of args "
-				"in join1 line" );
+			im_error( "global_balance", 
+				_( "bad number of args in join1 line" ) );
 			return( -1 );
 		}
 
@@ -628,11 +636,11 @@ process_line( SymbolTable *st, char *line )
 			!(arg2 = add_node( st, item[1] )) ||
 			!(join = add_node( st, item[2] )) )
 			return( -1 );
-		a = atof( item[3] );
-		b = atof( item[4] );
-		dx = atof( item[5] );
-		dy = atof( item[6] );
-		if( nitems == 8 ) 
+		a = g_ascii_strtod( item[3], NULL );
+		b = g_ascii_strtod( item[4], NULL );
+		dx = g_ascii_strtod( item[5], NULL );
+		dy = g_ascii_strtod( item[6], NULL );
+		if( nitems == 8 )
 			mwidth = atoi( item[7] );
 		else
 			mwidth = -1;
@@ -655,8 +663,8 @@ process_line( SymbolTable *st, char *line )
 		if( (nitems = break_items( line, item )) < 0 )
 			return( -1 );
 		if( nitems != 2 ) {
-			im_errormsg( "global_balance: bad number of args "
-				"in copy line" );
+			im_error( "global_balance", 
+				_( "bad number of args in copy line" ) );
 			return( -1 );
 		}
 
@@ -713,9 +721,9 @@ find_root( SymbolTable *st )
 	/* No root? Hot dang!
 	 */
 	if( !root ) {
-		im_errormsg( "im_global_balance: mosaic root not found "
-			"in desc file\n"
-			"is this really a mosaiced image?" );
+		im_error( "im_global_balance", 
+			_( "mosaic root not found in desc file\n"
+			"is this really a mosaiced image?" ) );
 		return( NULL );
 	}
 
@@ -724,28 +732,28 @@ find_root( SymbolTable *st )
 	 */
 	root->dirty = 1;
 	if( (notroot = im__map_table( st, is_root, NULL, NULL )) ) {
-		im_errormsg( "im_global_balance: more than one root" );
+		im_error( "im_global_balance", _( "more than one root" ) );
 		return( NULL );
 	}
 
 	return( root );
 }
 
-/* Split the Hist into lines, and parse each one.
+/* Walk history_list and parse each line.
  */
 int
 im__parse_desc( SymbolTable *st, IMAGE *in )
 {
-	char *nhist = im_strdup( st->im, in->Hist );
-	char *p = nhist;
-	char *q = nhist;
+	GSList *p;
 
-	if( !nhist ) 
-		return( -1 );
+	for( p = in->history_list; p; p = p->next ) {
+		GValue *value = (GValue *) p->data;
 
-	for( ; (q = im_break_token( p, "\n" )); p = q ) 
-		if( process_line( st, p ) ) 
+		assert( G_VALUE_TYPE( value ) == IM_TYPE_REF_STRING );
+
+		if( process_line( st, im_ref_string_get( value ) ) )
 			return( -1 );
+	}
 
 	/* Find root.
 	 */
@@ -1061,7 +1069,7 @@ find_image_stats( IMAGE *in, IMAGE *mask, Rect *area )
 
 #ifdef DEBUG
 	if( count == 0 )
-		im_warning( "global_balance: empty overlap!" );
+		im_warn( "global_balance", _( "empty overlap!" ) );
 #endif /*DEBUG*/
 
 	return( stats );
@@ -1188,8 +1196,8 @@ find_overlaps( JoinNode *node, SymbolTable *st )
 		/* Check for image.
 		 */
 		if( !node->im ) {
-			im_errormsg( "im_global_balance: unable to open "
-				"\"%s\"", node->name );
+			im_error( "im_global_balance", 
+				_( "unable to open \"%s\"" ), node->name );
 			return( node );
 		}
 		if( !node->trnim ) 
@@ -1522,8 +1530,8 @@ generate_trn_leaves( JoinNode *node, SymbolTable *st )
 		/* Check for image.
 		 */
 		if( !node->im ) {
-			im_errormsg( "im_global_balance: unable to open "
-				"\"%s\"", node->name );
+			im_error( "im_global_balance", 
+				_( "unable to open \"%s\"" ), node->name );
 			return( node );
 		}
 		if( node->trnim ) 
