@@ -1092,9 +1092,7 @@ im__lrmerge( IMAGE *ref, IMAGE *sec, IMAGE *out, int dx, int dy, int mwidth )
 
 	/* Prepare the output IMAGE.
 	 */ 
-	if( im_cp_desc( out, ref ) )
-		return( -1 );
-	if( im_append_Hist( out, sec ) )
+	if( im_cp_descv( out, ref, sec, NULL ) )
 		return( -1 );
 	out->Xsize = ovlap->oarea.width;
 	out->Ysize = ovlap->oarea.height;
