@@ -146,6 +146,94 @@ static im_function fzone_desc = {
 	zone_args 			/* Arg list */
 };
 
+/* Args for im_benchmark.
+ */
+static im_arg_desc benchmark_args[] = {
+	IM_INPUT_IMAGE( "in" ),
+	IM_OUTPUT_IMAGE( "out" )
+};
+
+/* Call im_benchmark via arg vector.
+ */
+static int
+benchmark_vec( im_object *argv )
+{
+	return( im_benchmarkn( argv[0], argv[1], 1 ) );
+}
+
+/* Description of im_benchmark.
+ */ 
+static im_function benchmark_desc = {
+	"im_benchmark", 		/* Name */
+	"do something complicated for testing", /* Description */
+	IM_FN_PIO,			/* Flags */
+	benchmark_vec, 			/* Dispatch function */
+	IM_NUMBER( benchmark_args ), 	/* Size of arg list */
+	benchmark_args 			/* Arg list */
+};
+
+/* Args for im_benchmark2.
+ */
+static im_arg_desc benchmark2_args[] = {
+        IM_INPUT_IMAGE( "in" ),
+        IM_OUTPUT_DOUBLE( "value" )
+};
+
+/* Call im_benchmark2 via arg vector.
+ */
+static int
+benchmark2_vec( im_object *argv )
+{
+        double f;
+
+        if( im_benchmark2( argv[0], &f ) )
+                return( -1 );
+
+        *((double *) argv[1]) = f;
+
+        return( 0 );
+}
+
+/* Description of im_benchmark2.
+ */
+static im_function benchmark2_desc = {
+        "im_benchmark2",                /* Name */
+        "do something complicated for testing", /* Description */
+        IM_FN_PIO,                      /* Flags */
+        benchmark2_vec,                 /* Dispatch function */
+        IM_NUMBER( benchmark2_args ),   /* Size of arg list */
+        benchmark2_args                 /* Arg list */
+};
+
+/* Args for im_benchmarkn.
+ */
+static im_arg_desc benchmarkn_args[] = {
+	IM_INPUT_IMAGE( "in" ),
+	IM_OUTPUT_IMAGE( "out" ),
+	IM_INPUT_INT( "n" )
+};
+
+/* Call im_benchmarkn via arg vector.
+ */
+static int
+benchmarkn_vec( im_object *argv )
+{
+	int n = *((int *) argv[2]);
+
+	return( im_benchmarkn( argv[0], argv[1], n ) );
+}
+
+/* Description of im_benchmarkn.
+ */ 
+static im_function benchmarkn_desc = {
+	"im_benchmarkn", 		/* Name */
+	"do something complicated for testing", /* Description */
+	IM_FN_PIO,			/* Flags */
+	benchmarkn_vec, 		/* Dispatch function */
+	IM_NUMBER( benchmarkn_args ), 	/* Size of arg list */
+	benchmarkn_args 		/* Arg list */
+};
+
 /* Args for im_grey.
  */
 static im_arg_desc grey_args[] = {
@@ -223,6 +311,9 @@ static im_function make_xy_desc = {
 /* Package up all these functions.
  */
 static im_function *other_list[] = {
+	&benchmark_desc,
+	&benchmark2_desc,
+	&benchmarkn_desc,
 	&eye_desc,
 	&grey_desc,
 	&feye_desc,
