@@ -31,7 +31,7 @@
 #endif /*HAVE_CONFIG_H*/
 #include <vips/intl.h>
 
-#include <stdlib.h>
+#include <cstdlib>
 
 #include <vips/vips.h>
 #include <vips/vipscpp.h>
@@ -40,6 +40,8 @@
 #include <dmalloc.h>
 #endif /*WITH_DMALLOC*/
 
+VIPS_NAMESPACE_START
+
 /* Refcounting stuff first.
  */
 
@@ -54,7 +56,7 @@ free_display( im_col_display *d )
 
 // Dupe an im_col_display
 static im_col_display *
-dup_display( im_col_display *in )
+dup_display( im_col_display *in ) throw( VError )
 {
 	im_col_display *out;
 
@@ -84,7 +86,7 @@ void VDisplay::refblock::cleanlut()
 void VDisplay::refblock::cleanref()
 {
 	if( disp && priv ) {
-		free_display( (im_col_display *) disp );
+		free_display( disp );
 		disp = 0;
 		priv = 0;
 	}
@@ -92,21 +94,20 @@ void VDisplay::refblock::cleanref()
 }
 
 // Get ready to write to disp
-void VDisplay::refblock::wready()
+void VDisplay::refblock::wready() throw( VError )
 {
 	cleanlut();
 	if( !priv ) {
-		disp = dup_display( (im_col_display *) disp );
+		disp = dup_display( disp );
 		priv = 1;
 	}
 }
 
 // Check that luts are up-to-date
-void VDisplay::refblock::cluts()
+void VDisplay::refblock::cluts() throw( VError )
 {
 	if( !luts )
-		if( !(luts = im_col_make_tables_RGB( NULL, 
-			(im_col_display *) disp )) )
+		if( !(luts = im_col_make_tables_RGB( NULL, disp )) )
 			verror();
 }
 
@@ -134,7 +135,7 @@ VDisplay &VDisplay::operator=( const VDisplay &a )
 	return( *this ); 
 }
 
-VDisplay::VDisplay( const char *name )
+VDisplay::VDisplay( const char *name ) throw( VError )
 {
 	// Search for a matching name in the VIPS colour list
         im_col_display *scr = im_col_display_name( name );
@@ -164,51 +165,23 @@ VDisplay::VDisplay( const char *name )
 
 VDisplay::VDisplay()
 {
-	// Just use SPARC
+	// Just use sRGB
 	ref = new refblock;
-	ref->disp = im_col_displays( 1 );
+	ref->disp = im_col_displays( 7 );
 }
 
-// Return references to fields in display. 
-char *VDisplay::name() 
-	{ ref->wready(); return( ((im_col_display*)ref->disp)->d_name ); }
-VDisplay::VDisplayType &VDisplay::type() 
-	{ ref->wready(); return( (VDisplayType&) 
-		((im_col_display*)ref->disp)->d_type ); }
-VDisplay::matrix &VDisplay::mat()
-	{ ref->wready(); return( ((im_col_display*)ref->disp)->d_mat ); }
+/*
+
+Setters and getters. We used to have a lot of code of the form:
+ 
 float &VDisplay::YCW()
 	{ ref->wready(); return( ((im_col_display*)ref->disp)->d_YCW ); }
-float &VDisplay::xCW()
-	{ ref->wready(); return( ((im_col_display*)ref->disp)->d_xCW ); }
-float &VDisplay::yCW()
-	{ ref->wready(); return( ((im_col_display*)ref->disp)->d_yCW ); }
-float &VDisplay::YCR()
-	{ ref->wready(); return( ((im_col_display*)ref->disp)->d_YCR ); }
-float &VDisplay::YCG()
-	{ ref->wready(); return( ((im_col_display*)ref->disp)->d_YCG ); }
-float &VDisplay::YCB()
-	{ ref->wready(); return( ((im_col_display*)ref->disp)->d_YCB ); }
-int &VDisplay::Vrwr()
-	{ ref->wready(); return( ((im_col_display*)ref->disp)->d_Vrwr ); }
-int &VDisplay::Vrwg()
-	{ ref->wready(); return( ((im_col_display*)ref->disp)->d_Vrwg ); }
-int &VDisplay::Vrwb()
-	{ ref->wready(); return( ((im_col_display*)ref->disp)->d_Vrwb ); }
-float &VDisplay::Y0R()
-	{ ref->wready(); return( ((im_col_display*)ref->disp)->d_Y0R ); }
-float &VDisplay::Y0G()
-	{ ref->wready(); return( ((im_col_display*)ref->disp)->d_Y0G ); }
-float &VDisplay::Y0B()
-	{ ref->wready(); return( ((im_col_display*)ref->disp)->d_Y0B ); }
-float &VDisplay::gammaR()
-	{ ref->wready(); return( ((im_col_display*)ref->disp)->d_gammaR ); }
-float &VDisplay::gammaG()
-	{ ref->wready(); return( ((im_col_display*)ref->disp)->d_gammaG ); }
-float &VDisplay::gammaB()
-	{ ref->wready(); return( ((im_col_display*)ref->disp)->d_gammaB ); }
-float &VDisplay::B()
-	{ ref->wready(); return( ((im_col_display*)ref->disp)->d_B ); }
-float &VDisplay::P()
-	{ ref->wready(); return( ((im_col_display*)ref->disp)->d_P ); }
+
+This should be split to separate setters/getters so we can exploit const. Too 
+annoying to do this on such a useless class (I'm certain no one used these 
+functions anyway), fix in vips8.
+
+ */
+
+VIPS_NAMESPACE_END
 
